from unittest import TestCase

from pylsqpack import DecoderStreamError, Encoder


class EncoderTest(TestCase):
    def test_decoder_stream_error(self):
        encoder = Encoder()
        with self.assertRaises(DecoderStreamError) as cm:
            encoder.feed_decoder(b"\x00")
        self.assertEqual(str(cm.exception), "lsqpack_enc_decoder_in failed")

    def test_encode_not_a_tuple(self):
        encoder = Encoder()
        stream_id = 0
        with self.assertRaises(ValueError) as cm:
            encoder.encode(stream_id, ["hello"])
        self.assertEqual(str(cm.exception), "the header must be a two-tuple")

    def test_encode_name_not_bytes(self):
        encoder = Encoder()
        stream_id = 0
        with self.assertRaises(ValueError) as cm:
            encoder.encode(stream_id, [(1, b"bar")])
        self.assertEqual(str(cm.exception), "the header's name and value must be bytes")

    def test_encode_value_not_bytes(self):
        encoder = Encoder()
        stream_id = 0
        with self.assertRaises(ValueError) as cm:
            encoder.encode(stream_id, [(b"foo", 1)])
        self.assertEqual(str(cm.exception), "the header's name and value must be bytes")

    def test_encode_too_long(self):
        encoder = Encoder()
        stream_id = 0
        with self.assertRaises(ValueError) as cm:
            encoder.encode(stream_id, [(bytes(4096), bytes(1))])
        self.assertEqual(str(cm.exception), "the header's name and value are too long")



import os.path
import sys

import setuptools
from wheel.bdist_wheel import bdist_wheel

extra_compile_args = []
include_dirs = [
    os.path.join("vendor", "ls-qpack"),
    os.path.join("vendor", "ls-qpack", "deps", "xxhash"),
]
if sys.platform == "win32":
    include_dirs.append(os.path.join("vendor", "ls-qpack", "wincompat"))
else:
    extra_compile_args = ["-std=c99"]


class bdist_wheel_abi3(bdist_wheel):
    def get_tag(self):
        python, abi, plat = super().get_tag()

        if python.startswith("cp"):
            return "cp39", "abi3", plat

        return python, abi, plat


setuptools.setup(
    ext_modules=[
        setuptools.Extension(
            "pylsqpack._binding",
            define_macros=[("Py_LIMITED_API", "0x03080000")],
            extra_compile_args=extra_compile_args,
            include_dirs=include_dirs,
            py_limited_api=True,
            sources=[
                "src/pylsqpack/binding.c",
                "vendor/ls-qpack/lsqpack.c",
                "vendor/ls-qpack/deps/xxhash/xxhash.c",
            ],
        ),
    ],
    cmdclass={"bdist_wheel": bdist_wheel_abi3},
)
